/**
 * @file dogm-cmd.c
 * libdogm
 *
 * This is a demostration module, that shows what this libdogm is useable for.
 *
 * LICENSE:
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * @author JORDACIJEVIC Serz <serz@jordacijevic.name>
 * @date 2008/05/19
 *
 * @version $Revision: 1.0$
 *
 * @todo Nothing to do. - Everything perfect! ;-)
 *
 * URL: $HeadURL: http://serz.jordacijevic.name/?mmenu=project&smenu=libdogm$
 *
 * Last Modified: $Author: Jordacijevic Serz$
 */

/*
 * -------------------------------------------------------------- includes --
 */

#define _GNU_SOURCE

/* std */
#include <avr/io.h>

/* local */
#include "dogm-cmd.h"
#include "dogm-data.h"

/*
 * --------------------------------------------------------------- defines --
 */

/*
 * -------------------------------------------------------------- typedefs --
 */

typedef struct s_date {
	uint16_t date_y;
	uint8_t date_m;
	uint8_t date_d;
	uint8_t time_h;
	uint8_t time_m;
	uint8_t time_s;
} date_t;

/*
 * --------------------------------------------------------------- globals --
 */

static date_t date = {'\0'};

/*
 * ------------------------------------------------------------- functions --
 */

static void print_date(void);

static void print_time_s(void);
static void print_time_m(void);
static void print_time_h(void);
static void print_date_d(void);
static void print_date_m(void);
static void print_date_y(void);

static void drive_time_s(void);
static void drive_time_m(void);
static void drive_time_h(void);
static void drive_date_d(void);
static void drive_date_m(void);
static void drive_date_y(void);

/**
 *
 * \brief date_set()
 *
 * This function initiates the date and time.
 *
 * \param dy year
 * \param dm month
 * \param dd day
 * \param th hour
 * \param tm minute
 * \param ts second
 *
 * \return always void
 * \retval void always
 *
 */
void date_set(uint16_t dy, uint8_t dm, uint8_t dd, uint8_t th, uint8_t tm, uint8_t ts)
{
	(date.date_y) = dy;
	(date.date_m) = dm;
	(date.date_d) = dd;

	(date.time_h) = th;
	(date.time_m) = tm;
	(date.time_s) = ts;

	print_date();
}

/**
 *
 * \brief date_addsec()
 *
 * increments the time and date by 1 second.
 *
 * \return always void
 * \retval void always
 *
 */
void date_addsec(void)
{
	drive_time_s();
}

static void print_date(void)
{
	dogm_cmd_disp(4);	// display on, cursor off, blink off
	dogm_cmd_disp_clr();	// clear display
	dogm_cmd_fset_dh(1);	// DH on
	dogm_cmd_dh_pos(0);	// 2/1 lines

	dogm_data_string("Time: ");
	dogm_data_char_xy(10, 0, ':');
	dogm_data_char_xy(13, 0, ':');

	dogm_data_string_xy(0, 1, "Date: ");
	dogm_data_char_xy(10, 1, '-');
	dogm_data_char_xy(13, 1, '-');

	print_time_s();
	print_time_m();
	print_time_h();

	print_date_d();
	print_date_m();
	print_date_y();
}

static void print_time_s(void)
{
	if ((date.time_s) < 10) {
		dogm_cmd_curser_set(14, 0);
		dogm_data_char('0');
		dogm_data_uint(date.time_s);
	} else {
		dogm_data_uint_xy(14, 0, (date.time_s));
	}
}

static void print_time_m(void)
{
	if ((date.time_m) < 10) {
		dogm_cmd_curser_set(11, 0);
		dogm_data_char('0');
		dogm_data_uint(date.time_m);
	} else {
		dogm_data_uint_xy(11, 0, (date.time_m));
	}
}

static void print_time_h(void)
{
	if ((date.time_h) < 10) {
		dogm_cmd_curser_set(8, 0);
		dogm_data_char('0');
		dogm_data_uint(date.time_h);
	} else {
		dogm_data_uint_xy(8, 0, (date.time_h));
	}
}

static void print_date_d(void)
{
	if ((date.date_d) < 10) {
		dogm_cmd_curser_set(14, 1);
		dogm_data_char('0');
		dogm_data_uint(date.date_d);
	} else {
		dogm_data_uint_xy(14, 1, (date.date_d));
	}
}

static void print_date_m(void)
{
	if ((date.date_m) < 10) {
		dogm_cmd_curser_set(11, 1);
		dogm_data_char('0');
		dogm_data_uint(date.date_m);
	} else {
		dogm_data_uint_xy(11, 1, (date.date_m));
	}
}

static void print_date_y(void)
{
	dogm_data_uint_xy(6, 1, (date.date_y));
}

static void drive_time_s(void)
{
	(date.time_s)++;
	(date.time_s) %= 60;		// 0-59 secounds
	if (!(date.time_s)) {
		drive_time_m();
	}
	print_time_s();
}

static void drive_time_m(void)
{
	(date.time_m)++;
	(date.time_m) %= 60;		// 0-59 minutes
	if (!(date.time_m)) {
		drive_time_h();
	}
	print_time_m();
}

static void drive_time_h(void)
{
	(date.time_h)++;
	(date.time_h) %= 24;		// 0-23 hours
	if (!(date.time_h)) {
		drive_date_d();
	}
	print_time_h();
}

static void drive_date_d(void)
{
	(date.date_d)++;
	switch (date.date_m) {
		case 1:					// Jan
		case 3:					// Mach
		case 5:					// May
		case 7:					// Jun
		case 8:					// Jul
		case 10:				// Oct
		case 12:				// Dec
			(date.date_d) %= 32;		// 31 days
			break;
		case 2:					// Feb
			if ((date.date_y) % 4) {
				(date.date_d) %= 29;		// 28 days
			} else {
				// every 4th Year
				(date.date_d) %= 30;		// 29 days
			}
			break;
		default:
			(date.date_d) %= 31;		// 30 days
			break;
	}
	if (!(date.date_d)) {
		(date.date_d)++;	// day can not be zero
		drive_date_m();
	}
	print_date_d();
}

static void drive_date_m(void)
{
	(date.date_m)++;
	(date.date_m) %= 13;		// 12 months
	if(!(date.date_m)) {
		(date.date_m)++;	// month can not be zero
		drive_date_y();
	}
	print_date_m();
}

static void drive_date_y(void)
{
	(date.date_y)++;
	print_date_y();
}

