/**
 * @file demo.c
 * libdogm
 *
 * This is a demostration module, that shows what this libdogm is useable for.
 *
 * LICENSE:
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * @author JORDACIJEVIC Serz <serz@jordacijevic.name>
 * @date 2008/05/19
 *
 * @version $Revision: 1.0$
 *
 * @todo Nothing to do. - Everything perfect! ;-)
 *
 * URL: $HeadURL: http://serz.jordacijevic.name/?mmenu=project&smenu=libdogm$
 *
 * Last Modified: $Author: Jordacijevic Serz; DI Michael Kramer$
 */

/*
 * -------------------------------------------------------------- includes --
 */

// 8MHz-Quarz
#define F_CPU		8000000UL
#define _GNU_SOURCE

/* std */
#include <avr/io.h>
#include <avr/iousb162.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <compat/deprecated.h>
#include <string.h>
#include <stdlib.h>

/* local */
#include "demo.h"
#include "demo-date.h"

#include "dogm-core.h"
#include "dogm-cmd.h"
#include "dogm-data.h"

/*
 * --------------------------------------------------------------- defines --
 */

/*
 * -------------------------------------------------------------- typedefs --
 */

/*
 * --------------------------------------------------------------- globals --
 */

/*
 * ------------------------------------------------------------- functions --
 */

int main(void)
{
	CLKPR = 0x80;
  	CLKPR = 0x00;
	DDRC|=1<<2; //PC2 controls Latch
    PORTC|=0b00000100;
	LCD_DOGM_DPORT = 0xff;			// set output
	LCD_DOGM_PORT = 0xff;			// clear LEDS
	SPCR = (1 << SPE) | (1 << MSTR) | (1<<SPR0);	// SPI Master
	// init the lcd
	dogm_core_set_remore(&(lcd_spi_write), &(lcd_set_rs), &(lcd_set_cs), &(_delay_us));
	dogm_cmd_init(16, 3, 1, &(_delay_ms));	// init x=16, y=3, 5V
	delay_s(2);

	write_all();			// write all chars to lcd
	delay_s(3);

	dogm_cmd_disp_clr();		// clear lcd
	dogm_data_string_xy(0, 0, "line 0: dogm163");
	dogm_data_string_xy(0, 1, "line 1: dogm163");
	dogm_data_string_xy(0, 2, "line 2: dogm163");
	delay_s(4);

	dogm_cmd_fset_dh(1);		// DH on
	dogm_cmd_dh_pos(0);		// 2/1 lines
	delay_s(4);

	dogm_cmd_dh_pos(1);		// 1/2 lines
	delay_s(4);

	dogm_cmd_fset_dh(0);		// DH off
	delay_s(4);

	date_set(2008, 5, 23, 23, 57, 0);
	while (1) {
		delay_s(1);
		date_addsec();
	}

	return (0);
}

/**
 *
 * \brief delay_s()
 *
 * delay in second
 *
 * \param sec is the delay in second
 *
 * \return always void
 * \retval void always
 *
 */
void delay_s(uint8_t sec)
{
	while(sec--) {
		_delay_ms(250);
		_delay_ms(250);
		_delay_ms(250);
		_delay_ms(250);
	}
}

/**
 *
 * \brief write_all()
 *
 * prints all chars
 *
 * \return always void
 * \retval void always
 *
 */
void write_all(void)
{
	unsigned char ichar[2] = {'\0'};
	ichar[0] = 0x00;
	do {
		dogm_data_char(ichar[0]++);
		_delay_ms(125);
	} while ( ichar[0] );
}

/**
 *
 * \brief lcd_spi_write()
 *
 * transfers the uchar to the SPI.
 *
 * \param hexwrite is the uchar you want to write
 *
 * \return always void
 * \retval void always
 *
 */
void lcd_spi_write(unsigned char hexwrite)
{
	_delay_us(1);
	SPDR = hexwrite;				// Daten nach Sender und starten
	// while(!(SPSR & (1<<SPIF)));			// warte bis Bit 1: fertig
	loop_until_bit_is_set(SPSR, SPIF);		// warte bis Bit 1: fertig
	_delay_us(1);
	hexwrite = SPDR;				// SPIF wider 0 (Pseudolesen)
}

/**
 *
 * \brief lcd_set_rs()
 *
 * with type 1 (true) you are selectiong the instruction register for writing.
 * With type 0 (false) you are selectiong the data register for writing. In SPI-Mode reading is not available.
 *
 * \param type can be 1 (true) or 0 (false)
 *
 * \return always void
 * \retval void always
 *
 */
void lcd_set_rs(uint8_t type)
{
	if (type) {
		// Command (LCD_CMD)
		cbi(LCD_DOGM_PORT, LCD_DOGM_RS);	// Reg Select: 0 => Instruction Reg (w)
	} else {
		// Data (LCD_DATA)
		sbi(LCD_DOGM_PORT, LCD_DOGM_RS);	// Reg Select: 1 => Data Reg (w)
	}
}

/**
 *
 * \brief lcd_set_cs()
 *
 * with 1 (ture) you can select and with type 0 (false) you are deselect the lcd.
 *
 * \param enable can be 0 or not 0
 *
 * \return always void
 * \retval void always
 *
 */
void lcd_set_cs(uint8_t enable)
{
	if (enable) {
		cbi(LCD_DOGM_PORT, LCD_DOGM_CSB);	// DOGM select = LOW
	} else {
		sbi(LCD_DOGM_PORT, LCD_DOGM_CSB);	// DOGM deselect = HIGH
	}
}

